#include <Arduino.h>
#include "temperature_LM35DZ.h"
#include "photo.h"
#include "Adafruit_AM2320.h"

#include "STM32LowPower.h"
#include <STM32RTC.h>
#include "motion.h"

#include <LecteurSD.h>
#include "tsl2591_lux.h"

#include "_Lora.h"
//#include "TSL2561.h"
#include "LoRa.h"

const int chipSelectPin = A7; // Mettre le numéro de broche correspondant à votre module SD
const char* data_file = "data.csv";
const char* motion_file = "motion.csv";
LecteurSD lecteurSD(chipSelectPin);


// /* Get the rtc (real time clock) object */
STM32RTC& rtc = STM32RTC::getInstance();

_Lora lora;

Adafruit_AM2320 am2320 = Adafruit_AM2320();


ICM20602 icm20602(0x69);

tsl2591_lux tsllux;


// Variables for RTC configurations
static uint8_t seconds = 0;
static uint8_t minutes = 49;
static uint8_t hours = 9;

static uint8_t weekDay = 4;
static uint8_t day = 4;
static uint8_t month = 2;
static uint8_t year = 24;

bool photo_shot = false;
//Time to sleep
/*

static uint8_t tts_seconds;
static uint8_t tts_minutes;
static uint8_t tts_hours;
*/

void mesure();

int i=0;


void interruptionicm()
{
  Serial.println("Start of interrupt icm");
  // disable interrupt
  detachInterrupt(D9);
  mesure();
  Serial.println("End of interrupt icm");
  // enable interrupt
  delay(250);
  attachInterrupt(D9, interruptionicm, RISING);
}

void interruptiontsl()
{
  Photo Camera;
  // turn on the camera
  Camera.power_on();
  delay(5000);
  // focus the camera
  Camera.focus(1000);
  // take a picture
  Camera.take_picture();
  delay(3000);
  // // turn off the camera
  Camera.power_off();
  Serial.println("Start of interrupt tsl");
  // disable interrupt
  detachInterrupt(D10);
  mesure();
  // enable interrupt
  delay(250);
  attachInterrupt(D10, interruptiontsl, FALLING);
  
  Serial.println("End of interrupt tsl");
}


void setup()
{
  Serial.println("Setup start");
  Serial.begin(115200);
  
  rtc.begin();
  rtc.setTime(hours, minutes, seconds);
  rtc.setDate(weekDay, day, month, year);
  Serial.println("fin rtc");

  icm20602.begin();
  icm20602.start_measurements();
  icm20602.setup_interrupt();

  attachInterrupt(D9, interruptionicm, RISING);
  Serial.println("fin icm");
  
  tsllux.configureSensor();
  attachInterrupt(D10, interruptiontsl, FALLING);
  Serial.println("fin tsl");

  pinMode(A1, INPUT);
  am2320.begin(); //initialiser le capteur de temperature AM2320
  Serial.println("fin am");
  

  Serial.println("Setup done");
}

/*
bool should_wake = false;

void alarm(void* data) {
  should_wake = true;
}

function to be in low power mode until noon not finished
void wait_for_noon(){
  // Configure first alarm in 2 second then it will be done in the rtc callback
  tts_hours = rtc.getHours();
  tts_minutes = rtc.getMinutes();
  tts_seconds = rtc.getSeconds();
  if(photo_shot == false)
  {
    tts_hours=(12-tts_hours)*pow(60,2);
    Serial.println("à tout à l'heure !");
  }
  else
  {
    tts_hours=(36-tts_hours)*pow(60,2); //1 hour => 3600 Epoch
    Serial.println("à demain !");
  }
  tts_minutes=(60-tts_minutes)*60; //1 min => 60 Epoch 
  tts_seconds=60-tts_seconds; //1 sec => 1 Epoch
  int TimeToSleep=tts_hours + tts_minutes + tts_seconds;
  Serial.println(String(TimeToSleep));
  
  rtc.setAlarmEpoch( rtc.getEpoch() + TimeToSleep);
  while (should_wake == false) {
    LowPower.deepSleep();
  } 
  should_wake = false;
}
*/

void mesure(){
  //int pin = A0;
  Serial.println("mesure");
  CTemperature T(A1); //initialize a temperature sensor in I2C
  //ordre 1) mvmt (1 ou 0) 2) rtc(epoch) 3)tempé interne (A1 askip) 4) tempé externe 5) humidité 6) lux 7) accel
  //";" entre chaque donnée + pas d'espace entre ";"
  // Serial.println("aaaaaaaaaaa"); //print the value read by the sensor

  float lux = tsllux.calculatelux();
  
  //Lecture des capteurs
  T.Read(); //print the temperature in °C
  
  int temp = am2320.readTemperature();
  delay(100); // d'après la datasheet, il faut attendre un court délai entre deux mesures. Sinon, la mesure ne se fait pas correctement.
  int hum = am2320.readHumidity();

  float x,y,z;
  icm20602.readAccelData(x, y, z);

  //mise en forme des données pour qu'elle puisse être inséré dans la BD
  String data = String(photo_shot); //déclarer data à chaque fois occupe de la mémoire ?
  data += ";" + String(int(rtc.getEpoch()));
  data += ";" + String(int(T.GetTemperature())) ;
  data += ";" + String(temp);
  data += ";" + String(hum);
  data += ";" + String(lux); //placeholder pour lux
  data += ";" + String(rtc.getDay()) + "/" + String(rtc.getMonth()) + "/" + String(rtc.getYear()) + ";" + " Heure : " + String(rtc.getHours())+" Minutes : " + String(rtc.getMinutes())+ " Secondes : "+ String(rtc.getSeconds()) ;
  data += ";" + String(x) + ";" + String(y) + ";" + String(z) + ";";
  Serial.println(data);
  lecteurSD.writeRow(data_file,";" +data);
  lora.send(data);


  //toggle built-in LED
  digitalWrite(LED_BUILTIN, !digitalRead(LED_BUILTIN));


}


void loop() 
{

}
